//+------------------------------------------------------------------+
//|                                                  Interchange.mqh |
//|                                           Copyright 2015, MigVRN |
//|                                                    Miguzov@bk.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, MigVRN"
#property link      "Miguzov@bk.ru"

#define BUFFER_MAX_SIZE 500000     //   
#define BUFFER_STEP     5000       //   
//+------------------------------------------------------------------+
//|     / 
//+------------------------------------------------------------------+
enum ENUM_CHART_EVENT_GLOBAL_VARIABLES
  {
   CHARTEVENT_SEND_DATA             =0x00000001,  //   
   CHARTEVENT_SEND_DATA_FINISH      =0x00000002,  //    
   CHARTEVENT_GET_DATA              =0x00000004,  //    
   CHARTEVENT_GET_DATA_FINISH       =0x00000008   //        
  };
//+------------------------------------------------------------------+
//|                       |
//+------------------------------------------------------------------+
struct GlobalVariablBuffer
  {
   double            buffer[BUFFER_MAX_SIZE];
  };
//+------------------------------------------------------------------+
//|       /      |
//+------------------------------------------------------------------+
class CInterchange
  {
private:
   GlobalVariablBuffer m_data;           //    
   double            m_tmp[BUFFER_STEP]; //  /

   string            m_prefix_name;      //    

   bool              m_flag_start_save;  //  
   bool              m_flag_start_load;  //  
   bool              m_flag_finish_save; //   
   bool              m_flag_finish_load; //   

   int               m_number_iteration;   // 
   ushort            m_chartevent_custom;  //   
   int               m_count_iteration;    //  

public:
   void              CInterchange(void):    m_prefix_name("Tmp"),
                                            m_flag_start_save(false),
                                            m_flag_start_load(false),
                                            m_flag_finish_save(false),
                                            m_flag_finish_load(false),
                                            m_number_iteration(0),
                                            m_chartevent_custom(1001) {};
   void             ~CInterchange(void) {};

   void              SetPrefixName(const string prefix) {m_prefix_name=prefix;} //     

   template<typename T>
   bool              StartSendData(T &Struct); //       

   template<typename T>
   bool              StartGetData(T &Struct);  //       

   template<typename T>
   bool              FinishGetData(T &Struct); //     

   bool              GetFlagSendDataStart() {return(m_flag_start_save);};   //    
   bool              GetFlagGetDataStart()  {return(m_flag_start_load);};   //    
   bool              GetFlagSendDataFinish() {return(m_flag_finish_save);}; //    
   bool              GetFlagGetDataFinish() {return(m_flag_finish_load);};  //    

   bool              OnEvent(const int id,const long &lparam,const double &dparam,const string &sparam);
   void              Destroy();                                         //          
private:
  };
//+------------------------------------------------------------------+
//|                       |
//+------------------------------------------------------------------+
template<typename T>
bool CInterchange::StartSendData(T &Struct)
  {
//---  ,     
   int Size_struct=sizeof(Struct);

//---    -  
   if(Size_struct>sizeof(m_data))
     {
      Print("!     !");
      return(false);
     }

//---      
   m_data=(GlobalVariablBuffer)Struct;

//---  
   m_flag_start_save  = true;
   m_flag_start_load  = false;
   m_flag_finish_save = false;
   m_flag_finish_load = false;
   m_number_iteration = 0;
   m_count_iteration  = 0;

//---   /     
   m_count_iteration=(int)MathCeil((double)Size_struct/(double(BUFFER_STEP*8.0)));

//---  
   EventChartCustom(ChartID(),m_chartevent_custom,(long)CHARTEVENT_SEND_DATA,0,m_prefix_name);

   return(true);
  }
//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+
template<typename T>
bool CInterchange::StartGetData(T &Struct)
  {
//---  ,     
   int Size_data=sizeof(Struct);

//---    -  
   if(Size_data>sizeof(m_data))
     {
      Print("!     !");
      return(false);
     }

//---  
   m_flag_start_save  = false;
   m_flag_start_load  = true;
   m_flag_finish_save = false;
   m_flag_finish_load = false;
   m_number_iteration = 0;
   m_count_iteration  = 0;

//---   /     
   m_count_iteration=(int)MathCeil((double)Size_data/(double(BUFFER_STEP*8.0)));

   return(true);
  }
//+------------------------------------------------------------------+
//|          |
//+------------------------------------------------------------------+
template<typename T>
bool CInterchange::FinishGetData(T &Struct)
  {
//---  ,     
   int Size_data=sizeof(Struct);

//---    -  
   if(Size_data>sizeof(m_data))
     {
      Print("!     !");
      return(false);
     }

   if(!m_flag_finish_load)
     {
      Print("!    -     !");
      return(false);
     }

   Struct=(T)m_data;

//---  
   m_flag_start_save  = false;
   m_flag_start_load  = false;
   m_flag_finish_save = false;
   m_flag_finish_load = false;
   m_number_iteration = 0;
   m_count_iteration  = 0;

   return(true);
  }
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
bool CInterchange::OnEvent(const int id,const long &lparam,const double &dparam,const string &sparam)
  {
   if(id!=CHARTEVENT_CUSTOM+m_chartevent_custom || sparam!=m_prefix_name)
      return(true);

   switch((ENUM_CHART_EVENT_GLOBAL_VARIABLES)lparam)
     {
      case CHARTEVENT_SEND_DATA:
        {
         if(!m_flag_start_save) return(false);

         m_number_iteration=(int)dparam;

         if(m_number_iteration>m_count_iteration) return(false);

         for(int i=0; i<BUFFER_STEP; i++)
           {
            if(GlobalVariableSet(m_prefix_name+"["+(string)i+"]",m_data.buffer[m_number_iteration*BUFFER_STEP+i])==0)
               return(false);
           }

         EventChartCustom(ChartID(),m_chartevent_custom,(long)CHARTEVENT_GET_DATA,(double)m_number_iteration,m_prefix_name);

         return(true);
        }
      //---    
      case CHARTEVENT_SEND_DATA_FINISH:
        {
         if(!m_flag_start_save) return(false);

         m_number_iteration=(int)dparam;

         if(m_number_iteration!=m_count_iteration) return(false);

         EventChartCustom(ChartID(),m_chartevent_custom,(long)CHARTEVENT_GET_DATA_FINISH,(double)m_number_iteration,m_prefix_name);

         m_number_iteration = 0;
         m_flag_start_save  = false;
         m_flag_finish_save = false;

         return(true);
        }
      case CHARTEVENT_GET_DATA:
        {
         if(!m_flag_start_load) return(true);

         if(m_number_iteration!=(long)dparam)
           {
            EventChartCustom(ChartID(),m_chartevent_custom,(long)CHARTEVENT_SEND_DATA,(double)m_number_iteration,m_prefix_name);
            return(false);
           }

         for(int i=0; i<BUFFER_STEP; i++)
           {
            if(!GlobalVariableGet(m_prefix_name+"["+(string)i+"]",m_data.buffer[m_number_iteration*BUFFER_STEP+i]))
               return(false);
           }

         m_number_iteration++;

         if(m_number_iteration>m_count_iteration)
           {
            m_number_iteration--;
            EventChartCustom(ChartID(),m_chartevent_custom,(long)CHARTEVENT_SEND_DATA_FINISH,(double)m_number_iteration,m_prefix_name);
           }
         else
           {
            EventChartCustom(ChartID(),m_chartevent_custom,(long)CHARTEVENT_SEND_DATA,(double)m_number_iteration,m_prefix_name);
           }

         return(true);
        }
      case CHARTEVENT_GET_DATA_FINISH:
        {
         if(!m_flag_start_load) return(true);

         m_number_iteration=0;
         m_flag_start_load  = false;
         m_flag_finish_load = true;

         return(true);
        }
     }
   return(false);
  }
//+------------------------------------------------------------------+
//|                |
//+------------------------------------------------------------------+
void CInterchange::Destroy()
  {
   GlobalVariablesDeleteAll(m_prefix_name);

   m_flag_start_save=false;
   m_flag_start_load=false;
   m_flag_finish_save=false;
   m_flag_finish_load=false;
   m_number_iteration=0;
  }
//+-------------------------------------------------------------------+  
